﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Threading;
using OvDotNet;

namespace OvApiDemo
{
    public partial class DemoForm : Form
    {
        public DemoForm()
        {
            InitializeComponent();
            //ovApi.OutputTraceMessages = true;
        }

        #region private member functions

        /// <summary>
        /// Update scrolling text box by adding text to the end
        /// </summary>
        /// <param name="txtBox">textbox object we are going to update</param>
        /// <param name="text">text to add to the box</param>
        private void AppendToTextBox(TextBox txtBox, string text)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new MethodInvoker(delegate() { AppendToTextBox(txtBox, text); }));
            }
            else
            {
                txtBox.AppendText(text);
                txtBox.SelectionStart = txtBox.Text.Length;
                txtBox.SelectionLength = 0;
                txtBox.ScrollToCaret();
            }
        }
        /// <summary>
        /// Update text box with new text
        /// </summary>
        /// <param name="txtBox">textbox object we are going to update</param>
        /// <param name="text">text to place into the box</param>
        private void UpdateTextBox(TextBox txtBox, string text)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new MethodInvoker(delegate() { UpdateTextBox(txtBox, text); }));
            }
            else
            {
                txtBox.Text = text;
                txtBox.ScrollToCaret();
                this.Refresh();
            }
        }

        /// <summary>
        /// Update emulation screen text
        /// </summary>
        /// <param name="text">text to assign to tb_screen</param>
        private void UpdateScreenText(string text)
        {
            UpdateTextBox(tb_screen, text);
        }
        /// <summary>
        /// Update the capture text 
        /// </summary>
        /// <param name="text">text to assign to tb_captureText</param>
        private void UpdateCaptureText(string text)
        {
            UpdateTextBox(tb_captureText, text);
        }
        /// <summary>
        /// Log capture text to our Capture Log text box
        /// </summary>
        /// <param name="text"></param>
        private void LogCaptureText(string text)
        {
            string s = "--------------------------------------";
            string t = s + DateTime.Now.ToString("hh:mm:ss.ff tt") + s;
            s = "\r\n" + s.PadRight(t.Length, '-');
            AppendToTextBox(tb_captureLog,"\r\n" + t + "\r\n" + text +s );
        }
        /// <summary>
        /// Add Event text to our event text box
        /// </summary>
        /// <param name="text"></param>
        private void LogEventText(string text)
        {
            AppendToTextBox(tb_events,DateTime.Now.ToString("HH:mm:ss.ff") + " " + text + "\r\n");
        }
        /// <summary>
        /// Add function key pressed to our event text box
        /// </summary>
        /// <param name="e"></param>
        private void LogEventFunctionKey(KeyEventArgs e)
        {
            string s = "";
            if (e.Alt)
                s += "Alt Key + ";
            if (e.Control)
                s += "Control Key + ";
            if (e.Shift)
                s += "Shift Key + ";
            s += e.KeyCode.ToString();
            LogEventText("FunctKey = " + s);
        }

        /// <summary>
        /// Updates the runtime properties text boxes in the "Properties Runtime Tab"
        /// </summary>
        private void UpdateRuntimeProperties()
        {
            tb_screenRows.Text = ovApi.Rows.ToString();
            tb_screenColumns.Text = ovApi.Columns.ToString();
            tb_currentRow.Text = ovApi.CrtRow.ToString();
            tb_currentColumn.Text = ovApi.CrtCol.ToString();
            tb_emulationType.Text = ovApi.CurrentEmulationType.ToString();
            tb_keyboardState.Text = ovApi.CurrentKeyboardState.ToString();
            tb_screenMode.Text = ovApi.CurrentScreenMode.ToString();
            tb_lastEventMessage.Text = ovApi.LastEventMessage;
        }

        private void timer_runtimeUpdate_Tick(object sender, EventArgs e)
        {
            timer_runtimeUpdate.Enabled = false;
            this.BeginInvoke(new MethodInvoker(delegate() { UpdateRuntimeProperties(); }));
        }
        /// <summary>
        /// Gets the screen as an array of strings.
        /// </summary>
        /// <returns>string array with each element the width of the screen</returns>
        private string[] GetScreenArray()
        {
            int screenRowCount = 0;                 //number of rows on the screen
            string s = "";
            s = ovApi.CrtQuery("HEIGHT");
            if (!string.IsNullOrEmpty(s))
            {
                int.TryParse(s, out screenRowCount);
            }
            if (screenRowCount == 0)
                screenRowCount = 24;
            int screenColumnCount = 0;              //number of columns on a screen row
            s = ovApi.CrtQuery("WIDTH");
            if (!string.IsNullOrEmpty(s))
            {
                int.TryParse(s, out screenColumnCount);
            }
            if (screenColumnCount == 0)
                screenColumnCount = 80;
            string[] ret = new string[screenRowCount];
            string ScreenText = ovApi.CrtGet(0, 0, screenRowCount * screenColumnCount);
            //
            //  Account for the differences between the emulators, 
            //  some trim trailing spaces and some don't!
            //
            StringReader r = new StringReader(ScreenText);
            int insertPtr = 0;
            string line = null;
            while ((line = r.ReadLine()) != null)
            {
                if (line.Length < screenColumnCount)
                {
                    line = line.PadRight(screenColumnCount);
                }
                ret[insertPtr++] = line;
            }
            return ret;
        }
        #endregion

        #region GUI Events

        #region Pure GUI events

        /// <summary>
        /// Update the text value for the selected function key
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboFuncKey_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboFuncKey.SelectedValue != null)
            {
                if (comboFuncKey.SelectedValue is string)
                {
                    tb_functKey.Text = (String)comboFuncKey.SelectedValue;
                }
                else
                {
                    tb_functKey.Text = "";
                }
            }
        }
        /// <summary>
        /// Display data error message for the input fields datagridview
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgv_inputFields_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            string s = e.Exception.Message;
            MessageBox.Show(s);
        }
        /// <summary>
        /// Clear various text boxes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_clearText_Click(object sender, EventArgs e)
        {
            tb_events.Text = "";
            tb_captureLog.Text = "";
            tb_captureText.Text = "";
            tb_command.Text = "";
            tb_password.Text = "";
            tb_account.Text = "";
            tb_screen.Text = "";
        }
        #endregion
        /// <summary>
        /// Radio button to set field text trimming to None.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rb_trimNone_CheckedChanged(object sender, EventArgs e)
        {
            if (rb_trimNone.Checked)
                ovApi.AutoTrimHostFields = OvFieldTextTrimming.None;
        }
        /// <summary>
        /// Radio button to set field text trimming to start of text.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rb_trimStart_CheckedChanged(object sender, EventArgs e)
        {
            if (rb_trimStart.Checked)
                ovApi.AutoTrimHostFields = OvFieldTextTrimming.StartOfField;
        }
        /// <summary>
        /// Radio button to set field text trimming to end of text
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rb_trimEnd_CheckedChanged(object sender, EventArgs e)
        {
            if (rb_trimEnd.Checked)
                ovApi.AutoTrimHostFields = OvFieldTextTrimming.EndOfField;

        }
        /// <summary>
        /// Radio button to set field text trimming to begining and end of text
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rb_trimStartAndEnd_CheckedChanged(object sender, EventArgs e)
        {
            if (rb_trimStartAndEnd.Checked)
                ovApi.AutoTrimHostFields = OvFieldTextTrimming.StartAndEndOfField;
        }
        /// <summary>
        /// Handle Event for enable text capture 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_enableCapture_Click(object sender, EventArgs e)
        {
            ovApi.DataCaptureEnable(true);
            btn_enableCapture.Enabled = false;
            btn_disableCapture.Enabled = true;
        }
        /// <summary>
        /// Handle Event for disable text capture
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_disableCapture_Click(object sender, EventArgs e)
        {
            ovApi.DataCaptureEnable(false);
            btn_enableCapture.Enabled = true;
            btn_disableCapture.Enabled = false;
        }

        /// <summary>
        /// Handle click event for visualize data button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_visualize_Click(object sender, EventArgs e)
        {
            VisualizeScreenLayout vis = new VisualizeScreenLayout(ovApi);
            vis.EnableRealTimeUpdate();
            if (vis.ShowDialog(this) == DialogResult.OK)
            {
            }
            vis.Dispose();
        }
        /// <summary>
        /// Form load event, assign the binding data sources after they are created.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DemoForm_Load(object sender, EventArgs e)
        {
            //
            //  Set the data sources for binding source objects
            //
            ovFieldInfoBindingSource.DataSource = ovApi.DataFields;
            ovInputFieldBindingSource.DataSource = ovApi.InputFields;
            string[] args = Environment.GetCommandLineArgs();
            if (args.Length < 2)
            {
                if (string.IsNullOrEmpty(ovApi.SessionFilename) && !ovApi.AutoStartOrAttach)
                {
                    LogEventText("Scanning for OV!");
                    timer_relinkToOv.Start();
                }
            }
            else
            {
                ovApi.HotLinkToOv(args);
            }
        }
        /// <summary>
        /// Handle click event for send text command
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_sendCmd_Click(object sender, EventArgs e)
        {
            ovApi.Text = tb_command.Text;
        }
        /// <summary>
        /// Handle click event for send password text command
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_sendPassword_Click(object sender, EventArgs e)
        {
            ovApi.Text = tbPassword.Text;
        }
        /// <summary>
        /// Handle Click event for send function key
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_functionKey_Click(object sender, EventArgs e)
        {
            ovApi.EmitFunctionKey((String)comboFuncKey.SelectedValue);
        }
        /// <summary>
        /// Handle Click event for the Login button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_login_Click(object sender, EventArgs e)
        {
            groupBox_blockModeLogin.Enabled = false;
            ovApi.InputFields[0].Text = tb_account.Text;
            ovApi.InputFields[1].Text = tb_password.Text;
            ovApi.EmitFunctionKey(tb_functKey.Text);
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DemoForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (!string.IsNullOrEmpty(ovApi.SessionFilename) && ovApi.AutoStartOrAttach)
            {
                ovApi.ShutDown();
            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_normalGUI_Click(object sender, EventArgs e)
        {
            ovApi.ApplicationRequest("NormalGUI");
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_simpleGUI_Click(object sender, EventArgs e)
        {
            ovApi.ApplicationRequest("SimpleGUI");
        }
        #endregion

        #region OV API Events

        private void ovApi_EmulationTypeActive(object sender, OvDotNet.OvEmulationTypeArgs e)
        {
            LogEventText(e.Emulation.ToString());
            ovApi.BindComboBoxToFunctionKeyList(comboFuncKey);
        }

        private void ovApi_KeyboardStateChanged(object sender, OvDotNet.OvKeyboardStateArgs e)
        {

        }

        private void ovApi_DataReceived(object sender, EventArgs e)
        {
            LogEventText("Data Received");
            UpdateScreenText(ovApi.Text);
            UpdateCaptureText(ovApi.DataCapture);
            timer_runtimeUpdate.Enabled = false;
            timer_runtimeUpdate.Enabled = true;
        }

        private void ovApi_DiaContextActive(object sender, OvDotNet.OvContextArgs e)
        {

        }

        private void ovApi_OnCaptureData(object sender, OvDotNet.OvCaptureArgs e)
        {
            LogCaptureText(e.Text);
        }

        private void ovApi_SessionStateChanged(object sender, OvDotNet.OvSessionStateArgs e)
        {

        }

        private void timer_relinkToOv_Tick(object sender, EventArgs e)
        {
            //
            //  Check if there is an instance of OV running
            //
            if (ovApi.OvWindowHandles.Length > 0)
            {
                timer_relinkToOv.Stop();                        //stop scanning
                ovApi.HotLinkToOv(ovApi.OvWindowHandles[0]);    //hot link to the first instance of OV
            }
        }
        private void ovApi_DataFieldsUpdate(object sender, EventArgs e)
        {
            bool enableFlag = false;
            string hostName = string.Empty;
            //
            //  Check if there is more then one input field on this screen
            // 
            if (ovApi.InputFields.Count > 1)
            {
                enableFlag = ovApi.InputFields[1].NoEcho;
                if (enableFlag)
                {
                    string[] scr = GetScreenArray();

                    string s = scr[1].Substring(47, 6);
                    if (s.Equals("System"))
                    {
                        hostName = scr[1].Substring(69).TrimEnd();
                    }

                }
            }
            groupBox_blockModeLogin.Enabled = enableFlag;
            tb_hostName.Text = hostName;
        }
        #endregion

        #region Session Control management

        private object m_controlLock = new object();
        private bool m_waitingForControl = false;

        private void btn_requestControl_Click(object sender, EventArgs e)
        {
            lock (this.m_controlLock)
            {
                bool gotControl = ovApi.RequestControl(true);
                if (gotControl)
                {
                    btn_requestControl.Enabled = false;
                    btn_releaseControl.Enabled = true;
                }
                else
                {
                    btn_requestControl.Enabled = false;
                    m_waitingForControl = true;
                }
            }
        }

        void ovApi_SessionOwnerChanged(object sender, OvDotNet.OvOwnerArgs e)
        {

        }


        private void btn_releaseControl_Click(object sender, EventArgs e)
        {
            ovApi.ReleaseControl();
            btn_requestControl.Enabled = true;
            btn_releaseControl.Enabled = false;
        }

        private void checkBox_careaboutControl_Click(object sender, EventArgs e)
        {
            bool honorControl = cb_honorSessionControl.Checked;
            ovApi.SimKeyboardLocked = honorControl;
            if (!honorControl)
            {
                btn_requestControl.Enabled = false;
                btn_releaseControl.Enabled = false;
                ovApi.ReleaseControl();                 //release control just incase we had it!
            }
            else
            {
                btn_requestControl.Enabled = true;
                btn_releaseControl.Enabled = false;
            }
        }

        #endregion

        private void ovApi_FunctionKeyPressed(object sender, KeyEventArgs e)
        {
            LogEventFunctionKey(e);
            if (e.Shift && e.KeyCode == Keys.F8)
            {
                e.Handled = true;
                LogEventText("Suppressed Shift-F8");
            }
            if (e.Alt && e.KeyCode == Keys.S && ovApi.CurrentScreenContext == "viewsys")
            {
                m_orginationKey = e;
                m_actionKey = TN6530FunctionKey.SHIFT_F1;
                bgw_keyBoard.RunWorkerAsync();
            }
            else
            {
                ovApi.FunctionKeyPressedDone(e);
            }
        }
        private KeyEventArgs m_orginationKey;
        private TN6530FunctionKey m_actionKey;
        private void bgw_keyBoard_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ovApi.EmitFunctionKey(TN6530FunctionKey.SHIFT_F1);
            ovApi.FunctionKeyPressedDone(m_orginationKey);
        }
        private void bgw_keyBoard_DoWork(object sender, DoWorkEventArgs e)
        {
            do
            {
                ovApi.EmitFunctionKey(m_actionKey);
                Thread.Sleep(4000);
                int i = ((int)m_actionKey) + 1;
                m_actionKey = (TN6530FunctionKey)i;
            } while (!(m_actionKey == TN6530FunctionKey.SHIFT_F16));
        }
    }
}
